/*******************************************************************************
*  TLine.cpp - bean for an encoded single trapped line
*
*  Berzerk/
*
*  Part of the BuckoSoft Network Monitor
*  Copyright(c) 2016 - BuckoSoft, Corp.
*
\*******************************************************************************/

#include "stdafx.h"
#include <iostream>
#include <boost/regex.hpp>
using namespace boost;

#include "TLine.h"

namespace berzerk {

TLine makeTLine(const string& s, const regex& rex, const regex& rexip) {
	boost::match_results<std::string::const_iterator> what;
	if (regex_search(s, what, rex)) {
		//if (DEBUG > 1)
		//	cout << "matched: " << s << endl;
		cout << "what[0]" << what[0] << endl;
		int match_start = s.find(what[0]);
		if (match_start < 0)
			match_start = 0;
		int match_length = what[0].length();
		int ip_start = 0;
		int ip_length = 0;
		if (regex_search(s, what, rexip)) {
			ip_start = s.find(what[0]);
			ip_length = what[0].length();
		}
		TLine tl = TLine(s, match_start, match_length, ip_start, ip_length);
		return(tl);
	}
	return(TLine());
}

string TLine::toXML() const {
	string s;
	if (!mLength && !ipLength) {
		s = line;
	} else if (mLength && !ipLength) {
		s = line.substr(0, mStart);
		s += "<span class=\"mlm\">";
		s += line.substr(mStart, mLength);
		s += "</span>";
		s += line.substr(mStart+mLength);
	} else if (ipLength && !mLength) {
		s = line.substr(0, ipStart);
		s += "<span class=\"mli\">";
		s += line.substr(ipStart, ipLength);
		s += "</span>";
		s += line.substr(ipStart+ipLength);
	} else {
		if (ipStart >= mStart && (ipStart+ipLength) < mLength) {		// overlapping segments
			s = "";
			SegType type = plain;
			int mark = 0;
			int i;
			for (i = 0; i<line.length(); i++) {
				if (i == ipStart) {
					s += makeSegment(mark, i, type);
					type = ip;
				} else if (type == ip) {
					if (i == ipStart+ipLength) {
						s += makeSegment(mark, i, ip);
						type = plain;
					}
				} else if (type == match && i == mStart+mLength) {
					s += makeSegment(mark, i, match);
					type = plain;
				} else if (type == plain && i >= mStart && i < mStart+mLength-1) {
					s += makeSegment(mark, i, plain);
					type = match;
				}
			}
			if (mark != i) {
				s += makeSegment(mark, i, type);
			}
		} else if (ipStart < mStart) {
			s = "";
			if (ipStart > 0)
				s = line.substr(0, ipStart);
			s += "<span class=\"mli\">";
			s += line.substr(ipStart, ipLength);
			s += "</span>";
			if (mStart > ipStart + ipLength)
				s += line.substr(ipStart + ipLength, mStart-(ipStart + ipLength));
			s += "<span class=\"mlm\">";
			s += line.substr(mStart, mLength);
			s += "</span>";
			if (mStart + mLength < line.length())
				s += line.substr(mStart + mLength);
		} else {
			s = "";
			if (mStart > 0)
				s = line.substr(0, mStart);
			s += "<span class=\"mlm\">";
			s += line.substr(mStart, mLength);
			s += "</span>";
			if (ipStart > mStart + mLength)
				s += line.substr(mStart + mLength, ipStart-(mStart + mLength));
			s += "<span class=\"mli\">";
			s += line.substr(ipStart, ipLength);
			s += "</span>";
			if (ipStart + ipLength < line.length())
				s += line.substr(ipStart + ipLength);

		}
	}
	return(s);
}

static char types[] = {'p', 'm', 'i' };

string TLine::makeSegment(int& mark, int index, SegType type) const {
	cout << "makeSegment: mark=" << mark << " index=" << index << " type=" << type << endl;
	string s;
	if (mark == index)
		return(s);
	s += "<span class=\"ml";
	s += types[type];
	s += "\">";
	s += line.substr(mark, index-mark);
	s += "</span>";
	mark = index;
	return(s);
}

}	// namespace berzerk
